

/*

% @(#)lu.c: version 2.1.1.2 created on 3/16/95 at 21:02:23

Copyright (c) 1993-1995 Daniel James Burnside
All rights reserved.
 
All copyrights are reserved by the author, except for
the explicit permissions granted herein.  Additional
uses may be allowed only by prior written permission
of the author.
 
Permission is granted for use of this work for
academic research purposes only. No commercial use of
any kind is permitted.  If you have downloaded a copy
of this work with the intent of a commercial
application, destroy all copies immediately.
Downloading is considered "making a copy" within the
meaning of the Copyright Act of 1978 (with amendments
in 1980 covering computer programs), and is an
exclusive right of the author.
 
Permission is granted to run this program, that is,
"perform the work".
 
Permission is granted to make verbatim copies and
derivative versions of this work. The copyright
notice, this permission notice, the author's name and
address, and the work's version number and date must
be preserved on all copies and versions.  A
translation into another computer language is a
derivative work.
 
Permission is NOT granted to market, or distribute,
copies or derivative versions of this work.
Specifically, but not exclusively, the work shall not
be made available through any electronic distribution
facility (e.g.: bulletin board, "email").
 
This work is provided on an "as is" basis.  The author
provides no warranty whatsoever, either express or
implied, regarding the work, including warranties with
respect to its merchantability or fitness for any
particular purpose.
 
The author can be reached at:
 
Daniel Burnside
School of Electrical Engineering
Cornell University
Ithaca, New York 14850
(607) 277-6613
danb@ee.cornell.edu

*/

#include "lu.h" 

static const char sccsid[] = "@(#)lu.c 2.1.1.2 3/16/95 21:02:23";  
/* SCCS version info */

/* ANSI style C */

#include <math.h>
#include <stdio.h>

#define SMALL 1.0e-20

void ludc(double **A, int n, int *index)

/* 
   Input:

        A is n x n double
        n is int

   Output: (index must have space allocated for it on entry.)

        index is length n integer 
        overwrites A with LU on output
*/

{
int i,j,jj,k;
double temp,y;

/************************************************/
for (j=1;j<=n;j++) {

  for (k=1;k<=j-1;k++) {
    temp=A[k][j];A[k][j]=A[index[k]][j];A[index[k]][j]=temp;
  }

  for (k=1;k<=j-1;k++) 
    for (i=k+1;i<=j-1;i++) 
      A[i][j]=A[i][j]-A[i][k]*A[k][j];

  for (k=1;k<=j-1;k++) 
    for (i=j;i<=n;i++) 
      A[i][j]=A[i][j]-A[i][k]*A[k][j];

  jj = j;
  y = fabs(A[jj][j]);
  for (k=j+1;k<=n;k++) 
    if (fabs(A[k][j]) > y) {
       jj = k; y = fabs(A[jj][j]);
    }

  index[j]=jj;
  for (k=1;k<=j;k++) {
    temp=A[j][k]; A[j][k]=A[jj][k]; A[jj][k]=temp;
  }

  y = A[j][j];
  if (fabs(y) < SMALL)
      (void) fprintf(stderr,"\nwarning: matrix nearly singular in ludc()\n");
  else if (fabs(y)!=0.0) {
    for (i=j+1;i<=n;i++) 
      A[i][j]=A[i][j]/y;
  }
}

} /* end function ludc() */


/* Should remove printfs and send back return code instead */

void lusb(double **LU, const int index[], double b[], 
          int mode, int n)

/* 
   Input:

        LU    is n x n double
        b     is n x 1 double
        index is n x 1 integer 
        n     is int

   Output: (index must have space allocated for it on entry.)

        overwrites b with soln
*/

{
int i,j;
double temp,sum;

/*
Note: the scramblings could probably be incorporated
into the loops directly, which may be faster.
Also, I can take advantage of the fact of leading 0's
in the rhs vector. This will save 1/2 the flops for
a matrix inversion.
*/

/*****************************************************/
if (mode==1) {           /* Ax = b */
/*****************************************************/
  
    for (i=1;i<=n-1;i++) { /* unscramble input b */
      temp = b[index[i]];
      b[index[i]] = b[i];
      b[i] = temp;
    }

    /* 
    solve Ly = b  b := y
    by forward substitution
    (y = L\b; b = y)
    */ 
        
    for (i=1;i<=n;i++) {
      sum=b[i];
      for (j=1;j<=i-1;j++)
        sum -= LU[i][j]*b[j];
      b[i]=sum;
    }
    
    /* 
    solve Ux = b  b := x
    by backward substitution
       x = U\y; b = x
    */ 
        
    for (i=n;i>=1;i--) {
      sum=b[i];
      for (j=i+1;j<=n;j++)
        sum -= LU[i][j]*b[j];
      b[i]=sum/LU[i][i];
    }
/*****************************************************/
} else if (mode==2) {           /* xA = b */
/*****************************************************/

    /* 
    solve yU = b  b := y
    equiv to U'y' = b', which is forward substitution
    */ 
    
    for (i=1;i<=n;i++) {
      sum=b[i];
      for (j=1;j<=i-1;j++) 
        sum -= LU[j][i]*b[j];
      b[i]=sum/LU[i][i];
    }

    /*
    solve xL = y  b := x
    equiv to L'y' = x', which is backward substitution
    */
    
    for (i=n-1;i>=1;i--) {
      sum=b[i];
      for (j=i+1;j<=n;j++)
        sum -= LU[j][i]*b[j];
    b[i]=sum;
    }

    for (i=n-1;i>=1;i--) {  /* unscramble output x */
      temp = b[i];
      b[i] = b[index[i]];
      b[index[i]] = temp;
    }

} else {
  (void) fprintf(stderr,"illegal mode in lusb()\n");
}

} /* end function lusb() */
