
/*

@(#)simplex.c: version 4.1.1.3 created on 3/16/95 at 21:04:35

Copyright (c) 1993-1995 Daniel James Burnside
All rights reserved.
 
All copyrights are reserved by the author, except for
the explicit permissions granted herein.  Additional
uses may be allowed only by prior written permission
of the author.
 
Permission is granted for use of this work for
academic research purposes only. No commercial use of
any kind is permitted.  If you have downloaded a copy
of this work with the intent of a commercial
application, destroy all copies immediately.
Downloading is considered "making a copy" within the
meaning of the Copyright Act of 1978 (with amendments
in 1980 covering computer programs), and is an
exclusive right of the author.
 
Permission is granted to run this program, that is,
"perform the work".
 
Permission is granted to make verbatim copies and
derivative versions of this work. The copyright
notice, this permission notice, the author's name and
address, and the work's version number and date must
be preserved on all copies and versions.  A
translation into another computer language is a
derivative work.
 
Permission is NOT granted to market, or distribute,
copies or derivative versions of this work.
Specifically, but not exclusively, the work shall not
be made available through any electronic distribution
facility (e.g.: bulletin board, "email").
 
This work is provided on an "as is" basis.  The author
provides no warranty whatsoever, either express or
implied, regarding the work, including warranties with
respect to its merchantability or fitness for any
particular purpose.
 
The author can be reached at:
 
Daniel Burnside
School of Electrical Engineering
Cornell University
Ithaca, New York 14850
(607) 277-6613
danb@ee.cornell.edu

*/

/*
      solves 
         min cx 

      subject to 
        Ax = b  
        x >= 0
*/

#      include "simplex.h" 
#      include "lu.h" 

#ifdef MALLOC_MODE
#      include "nrutil.h"
#endif

#      include <unistd.h>  /* _exit() */
#      include <stddef.h>  /* size_t */
#      include <stdio.h>   /* printf(), fflush() */
#      include <stdlib.h>  /* rand(), RAND_MAX */

#   define NRWMAX      251    /* max. no. of rows (constraints) */

#   define NCLMAX         (10*NRWMAX)
    /* max. no. of columns (variables) allowed in constraint mtx */

#     define CNSTDOUBLE 

#     define UNI ((float)rand()/RAND_MAX) /* uni real random no. on [0,1] */

#     define MAXPOS           1.0e+35	
				        
#     define MAXNEG          (-1.0e+35) 
				        

#     define OPTIMTOL        1.0e-10 
#     define RATIOTOL        1.0e-13  
#     define INFTOL          1.0e-08 
#     define REINVTOL        1.0e-24 

#     define NERR            500 

#     ifdef CNSTDOUBLE
        typedef double CNSTTYPE;
#     else
        typedef float CNSTTYPE;
#     endif

#     define cnstmt(A,B) (cnsttr[B][A]) 

#     define TRUE    1
#     define FALSE   0

    static const char sccsid[] = "@(#)simplex.c 4.1.1.3 3/16/95 21:04:35";  /* SCCS version info */

    static long int maxpiv[3];  
    static size_t typsiz; /* size_t typedef'd in stddef.h */
    static int nrow;
    int phase;  
    static int done; 
    static enum { 
      OPT, NPVEXC, UNBNDD, INFEAS, NRWEXC, NCLEXC
    } result;
    static int m;     
    static long npiv;		
    static int pivotc, pivotr;	
    static double pivelm;       
    static double cbar;		
    static int nclmax;
    static int canfre=FALSE;  
    static long int phasec[3];

#ifdef MALLOC_MODE

    static CNSTTYPE     **cnsttr;
    static double     **invers;	
    static    int        *sign;	
    static double       *price;	
				
				
    static     int      *inbss;	
    static     int     *trubss;	
    static     int      *basis;	
				
    static double         *lhs;	
    static double           *d;	
    static double           *c;	
    static double      *curcol;	

    static    int       *index; 
    static double      **baslu;	
    static double      *bfsnew;	
    static double      *dualnw;	
    static double      *column;

    static double      *curcl2;   

#else  

    static double   *basluP[NRWMAX + 2];
    static CNSTTYPE *cnsttP[NCLMAX];
    static double   *inverP[NRWMAX + 2];
    static double    **baslu;
    static CNSTTYPE **cnsttr;
    static double   **invers;

    static double   basluA[NRWMAX + 2][NRWMAX + 2];
    static CNSTTYPE cnsttA[  NCLMAX][NRWMAX + 1];
    static double   inverA[NRWMAX + 2][NRWMAX + 2];

    static    int inbss[NCLMAX];
    static    int trubss[NCLMAX]; 
    static    int sign[NRWMAX + 1];
    static double price[NRWMAX + 1];	
    static    int basis[NRWMAX + 1];	
    static double lhs[NRWMAX + 1];
    static double d[NCLMAX];
    static double c[NCLMAX];
    static double curcol[NRWMAX + 2];	
    static double bfsnew[NRWMAX + 1];
    static double dualnw[NRWMAX + 1];
    static double column[NRWMAX + 1]; 
    static    int  index[NRWMAX + 1]; 
    static double curcl2[NRWMAX + 1];

#endif 

    static int lastov = -1;  
    static int ncol;		
    static double curcst;	
    static int i, j, col, row;	

     void errmsg(const char *error_text,int errnum)
{
    void _exit(); /* see exit & _exit definition in man pages */

    (void) fprintf(stderr,"%s\n",error_text);

    (void) fprintf(stderr,"   ... exiting ...\n");
    _exit(1);
}

              static int pricol(int ncolin,int basmod)

{

  static double tmpcst;  
  static int pivotp;  
  static int optmal;  

  for (i = 0; i <= m; i++) price[i] = -invers[0][i + 1];

  if ((ncolin==1) & (maxpiv[phase]==1)) {

    if (basmod != 3) errmsg("basmod~=3 not handled yet in pricol()",0);

    if (lastov < 0) pivotc = ncol;
    else            pivotc = lastov+1; 
;
    cbar = d[pivotc - 1];
    for (i = 0; i <= m; i++) cbar -= price[i] * cnstmt(i,pivotc-1);
;
;
;
;

  } else {
    cbar = MAXPOS;
    for (col = 1; col <= ncol; col++) {
      if (inbss[col]==TRUE) continue; 
      tmpcst = d[col - 1];

      for (i = 0; i <= m; i++) tmpcst -= price[i] * cnstmt(i,col-1);
      if (cbar > tmpcst) {
         cbar = tmpcst; 
         pivotc = col;
      }
    } 
  } 

;

  if (cbar > -OPTIMTOL) {
    optmal = TRUE;
;
;
;
;
  } else {
    optmal = FALSE;
;
;
;
;
  }

  return(optmal);

} 

                  static int ratiot(void)

{
 static double perr,derr;
 static int cg;

  double ratio, minratio;	
  int rand(void);  

  for (i = 1; i <= m+1; i++) {	
    curcol[i] = 0.0;		

      for (j = 0; j <= m; j++)
        curcol[i] += invers[i][j + 1] * cnstmt(j,pivotc - 1);
  }

      if (0) { 

;

      for (j=1; j<=m+1; j++) {
         for (i=1; i<=m+1; i++) {
          if (basis[j-1]-1 < 0)  
            if (i == j) baslu[i][j] = 1;
            else        baslu[i][j] = 0;
          else
            baslu[i][j] = cnstmt(i-1,basis[j-1]-1);
        }
      }

      ludc(baslu,m+1,index);

      for (i=1; i<=m+1; i++) curcl2[i] = cnstmt(i-1,pivotc-1);
      lusb(baslu,index,curcl2,1,m+1);

      for (j=1; j<=m+1; j++) {
;
;

      curcol[j] = curcl2[j];
      }

  }

  curcol[0] = cbar;		
  pivotr = -1;
  minratio = MAXPOS;
  for (i = 0; i <= m; i++) {	
    if (curcol[i + 1] > RATIOTOL) {
      ratio = invers[i + 1][0] / curcol[i + 1];
      if (minratio > ratio) {   
	minratio = ratio;
	pivotr = i;
	pivelm = curcol[i + 1];
      } else {			
        if (minratio == ratio && pivelm < curcol[i + 1]) {

          float uni;
          uni = UNI;
          if (uni<.5) {

            pivotr = i; 
            pivelm = curcol[i + 1];
          }
        } 
      } 
    } 
  } 

  if (pivotr == -1) 
     {
;
;
;
;
;
;
     for (i=0;i<=m;i++) {
;
;
     }
;
;
;
;
;
;
;
;
       {
       bsserr(&perr,&derr);
;
;
;
       }
     cg = TRUE;		
     }
  else
    cg = FALSE;

  return(cg);
}				

                 static void pivot2(void)

{

  static double  *Ppivr0; 
  static double  *Ppivr;  
  static double  *Pcurc;  
  static double **PPinvr; 
  static double  *Pinvr;  
  static double  *Ppivrl; 
  static double  *Pcurcl; 
  static int mm;

  mm = nrow;

;
  i = basis[pivotr];
  if (i >= 1) inbss[i] = FALSE;  
  basis[pivotr] = pivotc;
  i = pivotc;
  if (i >= 1) inbss[i] = TRUE;

  pivotr++;
  Ppivr0 = invers[pivotr];
  Ppivrl = &Ppivr0[mm]; 

  for (Ppivr = Ppivr0; Ppivr <= Ppivrl; Ppivr++) *Ppivr /= pivelm;

  PPinvr = invers;
  Pcurc = curcol;

  for (Pcurcl = &curcol[pivotr]; Pcurc <  Pcurcl; Pcurc++)
    for (Pinvr = *PPinvr++,Ppivr = Ppivr0; Ppivr <= Ppivrl; Pinvr++,Ppivr++) 
       *Pinvr -= *Ppivr * *Pcurc;

  PPinvr++; Pcurc++;

  for (Pcurcl = &curcol[mm] ; Pcurc <= Pcurcl; Pcurc++)
    for (Pinvr = *PPinvr++,Ppivr = Ppivr0; Ppivr <= Ppivrl; Pinvr++,Ppivr++) 
       *Pinvr -= *Ppivr * *Pcurc;

  curcst = -invers[0][0];
  npiv++;
  phasec[phase]++;
  pivotr--;

  return;

  for (j = 0; j <= m+1; j++)
    invers[pivotr + 1][j] /= pivelm;

  for (i = 0; i <= m+1; i++) {
    if (i - 1 != pivotr) {
      for (j = 0; j <= m+1; j++)
	invers[i][j] -= invers[pivotr + 1][j] * curcol[i];
    }
  }

} 

                 static void pivot(void)

{

;
  i = basis[pivotr];
  if (i >= 1) inbss[i] = FALSE;  
  basis[pivotr] = pivotc;
  i = pivotc;
  if (i >= 1) inbss[i] = TRUE;
  for (j = 0; j <= m+1; j++)
    invers[pivotr + 1][j] /= pivelm;
  for (i = 0; i <= m+1; i++) {
    if (i - 1 != pivotr) {
      for (j = 0; j <= m+1; j++)
	invers[i][j] -= invers[pivotr + 1][j] * curcol[i];
    }
  }
  curcst = -invers[0][0];
  npiv++;
  phasec[phase] += (long)1;
} 

                static void cngphs(void)

{

  int b; 

  phase = 2;

  for (j = 0; j < ncol; j++)	
    d[j] = c[j];
  for (j = 0; j <= m+1; j++) {
    invers[0][j] = 0.0;
    for (i = 0; i <= m; i++) {
      b = basis[i];		
      if (b >= 1)		
	invers[0][j] -= c[b - 1] * invers[i + 1][j];
    }
  }
  curcst = -invers[0][0];
}				

                static void allocm(void)

{

#ifdef MALLOC_MODE

  int nbyt = 0; 
  int nbytot = 0; 

;
;

  typsiz = sizeof(CNSTTYPE);
  nbyt = nclmax*nrow*typsiz; 
  nbytot += nbyt;
;
;

#  ifdef CNSTDOUBLE
;
      cnsttr = dblmtx(0,nclmax-1,0,nrow-1);
#  else
;
      cnsttr = fltmtx(0,nclmax-1,0,nrow-1);
#  endif

  nbyt = (nrow+1)*(nrow+1)*8; 
  nbytot += nbyt;
;
;

  nbyt = nrow*nrow*8; 
  nbytot += nbyt;
;
;

;
;

  invers    = dblmtx(0,nrow,0,nrow);
  sign      = intvec(0,nrow-1);
  price     = dblvec(0,nrow-1);
  basis     = intvec(0,nrow-1);
  trubss    = intvec(1,nclmax); 
  inbss     = intvec(1,nclmax);
  lhs       = dblvec(0,nrow-1);
  d         = dblvec(0,nclmax-1);
  c         = dblvec(0,nclmax-1);
  curcol    = dblvec(0,nrow);

  baslu      = dblmtx(1,nrow,1,nrow);
  bfsnew     = dblvec(1,nrow);
  dualnw     = dblvec(1,nrow);
  column     = dblvec(1,nrow); 
  index      = intvec(1,nrow); 

  curcl2     = dblvec(1,nrow);

  for (i = 1; i <= nclmax; i++) inbss[i]  = FALSE;
  for (i = 1; i <= nclmax; i++) trubss[i] = i;

  canfre = TRUE;  

;

#else
  for (i = 0; i <= NRWMAX + 2; i++) basluP[i] = &basluA[i][0];
  for (i = 0; i <=   NCLMAX; i++) cnsttP[i] = &cnsttA[i][0];
  for (i = 0; i <= NRWMAX + 2; i++) inverP[i] = &inverA[i][0];
  baslu   = &basluP[0];
  cnsttr  = &cnsttP[0];
  invers  = &inverP[0];

  for (i = 1; i <= NCLMAX; i++) inbss[i]  = FALSE;
  for (i = 1; i <= NCLMAX; i++) trubss[i] = i;
#endif

} 

               void freeup(void)  

{

#   ifdef MALLOC_MODE

  if (canfre) {
    canfre = FALSE;

#   ifdef CNSTDOUBLE
      fdblmt(cnsttr,0,nclmax-1,0,nrow-1);
#   else
      ffltmt(cnsttr,0,nclmax-1,0,nrow-1);
#   endif

    fdblmt(invers,0,nrow,0,nrow);
    fintvc(sign,0,nrow-1);
    fdblvc(price,0,nrow-1);
    fintvc(basis,0,nrow-1);
    fintvc(trubss,1,nclmax); 
    fintvc(inbss,1,nclmax);
    fdblvc(lhs,0,nrow-1);
    fdblvc(d,0,nclmax-1);
    fdblvc(c,0,nclmax-1);
    fdblvc(curcol,0,nrow);

    fdblmt(baslu,1,nrow,1,nrow);
    fdblvc(bfsnew,1,nrow);
    fdblvc(dualnw,1,nrow);
    fdblvc(column,1,nrow);
    fintvc(index,1,nrow);

    fdblvc(curcl2,1,nrow);

    (void)fprintf(stderr,"Freed malloc-ed memory in "__FILE__ "\n");

  } 

#   endif

} 

                     void reinvr(int nreinv[])

{

  for (j=1; j<=m+1; j++)
    for (i=1; i<=m+1; i++) {
      if (basis[j-1]-1 < 0)   
        if (i == j) baslu[i][j] = 1;
        else        baslu[i][j] = 0;
      else
        baslu[i][j] = cnstmt(i-1,basis[j-1]-1);
    }

  ludc(baslu,m+1,index);

  for (j=1; j<=m+1;j++) {
    for (i=1; i<=m+1;i++) column[i] = 0.0;
    column[j] = 1.0; 
    lusb(baslu,index,column,1,m+1);
    for (i=1; i<=m+1;i++) invers[i][j]=column[i];
  }

  for (i=1; i<=m+1; i++) bfsnew[i] = lhs[i-1];

  lusb(baslu,index,bfsnew,1,m+1);

  for (i=1; i<=m+1; i++) 
    invers[i][0] = bfsnew[i];  

  for (i=1; i<=m+1; i++) dualnw[i] = -d[basis[i-1]-1];
  lusb(baslu,index,dualnw,2,m+1);
  for (i=1; i<=m+1; i++) invers[0][i] = dualnw[i];

  nreinv[phase]++;  

} 

        static int setup(int basmod,int nrowin,
                         int ncolin,double *cost,
                         double *b,double *A,
                         double *basin,int augmod,
                         int nreinv[],double nclfac)

{

  nrow = nrowin;
# ifndef MALLOC_MODE
     if (NRWMAX < nrowin) {
       fprintf(stderr,"Fatal error in setup(): ");
       fprintf(stderr,"NRWMAX < nrowin\n");
       fprintf(stderr,"NRWMAX: %d\n",NRWMAX);
       fprintf(stderr,"nrowin: %d\n",nrowin);
       return(NRWEXC); 
     } 
# endif

  m = nrow - 1;		

;
  if (basmod!=1 && basmod!=3) {
     ncol = ncolin;

       if (nclfac) 
         nclmax = (int )(nclfac*m); 
       else
         nclmax = ncolin;  
#      ifndef MALLOC_MODE
         nclmax = (nclmax < NCLMAX) ? nclmax : NCLMAX; 
#      endif
;
;
;
       if (nclmax < ncolin) {
          fprintf(stderr,"Fatal error in setup(): ");
          fprintf(stderr,"nclmax < ncolin\n");
          fprintf(stderr,"nclfac: %e\n",nclfac);
          fprintf(stderr,"m:      %d\n",m);
          fprintf(stderr,"ncolin: %d\n",ncolin);
          return(NCLEXC); 
        } 

   } 

  if (nclmax==0) errmsg("nclmax=0 in setup()",0);

;
;
;

  if (basmod==0) { 

     if (ncol > nclmax) {		
        done = TRUE;
        result = NCLEXC;
     } else {
        allocm();
        for (j = 0; j < ncol; j++) 
            c[j] = cost[j];
        for (i = 0; i <= m; i++) {

          if (b[i] < 0) {
            sign[i] =    -1;
            lhs[i]  = -b[i];
          } else {
            sign[i] =     1;
            lhs[i]  =  b[i];
          }

        } 

        for (j = 0; j < ncol; j++)
          for (i = 0; i <= m; i++){
            cnstmt(i,j) = *(A+i+(j*nrowin)) * sign[i];

          }

        for (i = 0; i <= nrow; i++)
            for (j = 0; j <= nrow; j++)
                invers[i][j] = 0.0; 

        for (i = 1; i <= nrow; i++)	
          invers[i][i] = 1.0;

        for (i = 0; i <= m; i++) {
          invers[i + 1][0] = lhs[i];	
          invers[0][0] -= lhs[i];	
        }
        for (i = 0; i <= m; i++)	
          basis[i] = -i;

        for (col = 1; col <= ncol; col++) {

          d[col - 1] = 0.0;
          for (row = 0; row <= m; row++)
    	    d[col - 1] -= cnstmt(row,col - 1);
        }

    phase = 1; 
    npiv = (long)0;
  }

  } else if (basmod==1) {  

    if (ncol+ncolin > nclmax) { 
       done = TRUE;
       result = NCLEXC;
    } else {

;

      for (j = ncol; j < ncol+ncolin; j++) 
          c[j] = cost[j-ncol]; 

      for (j = 0; j < ncolin; j++)
        for (i = 0; i <= m; i++)
          cnstmt(i,j+ncol) = *(A+i+(j*nrow)) * sign[i];

       for (j = ncol; j < ncol+ncolin; j++) 
          d[j] = c[j]; 

;
       ncol += ncolin;   
;
       phase = 2; 

    } 

  } else if (basmod==2) {  

    if (ncol > nclmax) {         
      done = TRUE;
      result = NCLEXC;
    } else {
      allocm();
      for (j = 0; j < ncol; j++) 
          c[j] = cost[j];
      for (i = 0; i <= m; i++) {

        if (b[i] < 0) {
           sign[i] = -1;
           lhs[i]  = -b[i];
        } else {
           sign[i] =   1;
           lhs[i]  =  b[i];
        } 
      } 

       for (j = 0; j < ncol; j++)
         for (i = 0; i <= m; i++)
           cnstmt(i,j) = *(A+i+(j*nrowin)) * sign[i];

       for (j = 0; j <= m; j++) {
          basis[j] = (int)basin[j];

       }

      for (j = 0; j < ncol; j++)  d[j] = c[j];

      reinvr(nreinv); 
      phase = 2; 
      npiv = (long)0;
    } 

  } else if (basmod==3) {  

     static int over,augm;
     static int totcol = 0;       

    if (ncolin + m > nclmax) {         
      done = TRUE;
      result = NCLEXC;
    } else {
;
;
;
;
      if (ncol+ncolin > nclmax) {  
        over = ncol+ncolin-nclmax;  
        augm = nclmax - ncol;
        if (!totcol) {
          totcol=nclmax; 
;
;
;
        }
      } else {                               
        over = 0;
        augm = ncolin;               
      }

;
;
;
;

      for (col = 0; col < augm; col++)
        for (i = 0; i <= m; i++)
          cnstmt(i,col+ncol) = *(A+i+(col*nrow)) * sign[i];

      for (j = ncol; j < ncol+augm; j++) 
         c[j] = cost[j-ncol]; 

      for (j = augm; j < augm+over; j++) {

         do if (++lastov > nclmax-1) lastov -= nclmax; 
         while (inbss[lastov+1]);

;
         trubss[lastov+1] = ++totcol;
         for (i = 0; i <= m; i++) 
            cnstmt(i,lastov) = *(A+i+(j*nrow)) * sign[i];
         c[lastov] = cost[j];  
      } 

;
       if (over) ncol = nclmax;
       else      ncol = ncol+ncolin;
;

       for (j = 0; j < ncol; j++)  d[j] = c[j];

       phase = 2; 

    } 

  } else {

  } 

    curcst = -invers[0][0];

;

   return(0); 
}  

        void bsserr(double *perr,double *derr)

{

double sum;

   *perr = 0.0;
   for (i = 0; i <= m; i++) {             
      sum = 0.0;
      for (j = 0; j <= m; j++) {            
        if (basis[j] < 0) {  
          if (-i == basis[j]) sum += invers[j+1][0]; 
        } else {
          sum += cnstmt(i,basis[j]-1) * invers[j+1][0]; 
;
;
;
;
        }
      }
;
      sum -= lhs[i];
      *perr += sum * sum;
   }
   *perr = *perr/(double)nrow;  

   *derr = 0.0;
   for (j = 0; j <= m; j++) {             
      sum = 0.0;
      if (basis[j] < 0) {  
        for (i = 0; i <= m; i++) 
          if (-i == basis[j])
            sum -= invers[0][i+1];
      } else {
        for (i = 0; i <= m; i++) sum -= invers[0][i+1] * cnstmt(i,basis[j]-1);
      }
      sum -= d[basis[j]-1];
      *derr += sum * sum;
   }
   *derr = *derr/(double)nrow;  

}  

     int simplx(int basmod,long *npivrt,
                double *bfs,double *bssret,
                double *y,int nrowin,int ncolin,
                double *cost,double *b,double *A,
                double *basin,int augmod,
                long mxpvin, int nreinv[],
                double nclfac)

{
  double perr,derr;
  int extcod;

  done = FALSE;

  maxpiv[1] = (long)(100 * ncolin);

  if (!mxpvin) {
      maxpiv[2] = (long)100 * (long)ncolin;
;
  } else {
      maxpiv[2] = mxpvin;
;
  }

  nreinv[1]  = nreinv[2]  = 0;
  phasec[1] = phasec[2] = (long)0;

  if (!phase) { 
;
;
  }

  if (extcod=setup(basmod,nrowin,ncolin,cost,b,A,basin,augmod,nreinv,nclfac))
    return(extcod);

;
;
;
;
;
;
;

  while (phasec[phase] < maxpiv[phase] && 
         !done && 
         (curcst > MAXNEG || phase == 1)) {

;
    if (!pricol(ncolin,basmod)) { 
;
      if (!ratiot()) {  
        pivot2(); 
;
;
;
;
        if (npiv % (long)NERR == (long)0) {
          bsserr(&perr,&derr);
;
;
;
          if (perr>REINVTOL||derr>REINVTOL){
;
;
;
;
            reinvr(nreinv); 
            bsserr(&perr,&derr);
;
;
;
          } 
        } 
      } else {
        done = TRUE;
        result = UNBNDD;
;
      } 

      continue;  
    }  
    if (phase == 1) { 
      if (curcst > INFTOL) {
	done = TRUE;
	result = INFEAS;
;
      } else {

	cngphs();
      }
    } else {   
      done = TRUE;
      result = OPT;
    }

;
;
;
;

  }   

;
;

  if (curcst <= MAXNEG && phase == 2) {
;
    result = UNBNDD;
;
  }
  for (i=1; i<=2; i++) {
    if (phasec[i] >= maxpiv[i]) {
;
;
;
;
      result = NPVEXC; 
    } 
  }

  if (result == INFEAS)
;
  if (result == NRWEXC)
;
  if (result == NCLEXC)
;

  if (result != OPT && result != NPVEXC) { 

;

  } else {  

   for (i = 0; i <= m; i++) {
    bssret[i] = (double) trubss[basis[i]];
    bfs[i]       = (double) invers[i + 1][0];
    if(b[i] < 0.0)    
      y[i] = -invers[0][i+1];   
    else
      y[i] = invers[0][i+1];   
    }

;
    *npivrt = npiv;

;
;
;

  } 

   if (!augmod) freeup();  

   return(result);  

} 
