
/* Mex file driver for C function simplex.c */

/*
  @(#)simplexM.c: version 2.1.1.3 created on 3/16/95 at 21:07:08

Copyright (c) 1993-1995 Daniel James Burnside
All rights reserved.
 
All copyrights are reserved by the author, except for
the explicit permissions granted herein.  Additional
uses may be allowed only by prior written permission
of the author.
 
Permission is granted for use of this work for
academic research purposes only. No commercial use of
any kind is permitted.  If you have downloaded a copy
of this work with the intent of a commercial
application, destroy all copies immediately.
Downloading is considered "making a copy" within the
meaning of the Copyright Act of 1978 (with amendments
in 1980 covering computer programs), and is an
exclusive right of the author.
 
Permission is granted to run this program, that is,
"perform the work".
 
Permission is granted to make verbatim copies and
derivative versions of this work. The copyright
notice, this permission notice, the author's name and
address, and the work's version number and date must
be preserved on all copies and versions.  A
translation into another computer language is a
derivative work.
 
Permission is NOT granted to market, or distribute,
copies or derivative versions of this work.
Specifically, but not exclusively, the work shall not
be made available through any electronic distribution
facility (e.g.: bulletin board, "email").
 
This work is provided on an "as is" basis.  The author
provides no warranty whatsoever, either express or
implied, regarding the work, including warranties with
respect to its merchantability or fitness for any
particular purpose.
 
The author can be reached at:
 
Daniel Burnside
School of Electrical Engineering
Cornell University
Ithaca, New York 14850
(607) 277-6613
danb@ee.cornell.edu

*/

#include "simplex.h"

#include <stdlib.h> /* srand */

#include "mex.h"   /* has a #define printf mexPrintf line */
#undef printf  /* use the stdio routine */
#include <stdio.h> /* printf */

#define VPRINTF
#undef VPRINTF  

#ifdef VPRINTF   
;
#else
;
#endif

static char sccsid[] = "@(#)simplexM.c 2.1.1.3 3/16/95 21:07:08";  

extern int phase;		

#define C_IN      prhs[0]
#define A_IN      prhs[1]
#define B_IN      prhs[2]
#define MODE_IN   prhs[3]   
#define AUGMODE   prhs[4]   
#define MAXPIV    prhs[5]   
#define NCLFAC_IN prhs[6] 
#define BASIS_IN  prhs[7]   

#define BASIS_OUT     plhs[0]
#define BFS_OUT       plhs[1]
#define Y_OUT         plhs[2]
#define N_OUT         plhs[3]
#define EXITCODE      plhs[4]
#define NREINVERT_OUT plhs[5]

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
        double         *y;  
        double         *n;  
        long           nI;  
        double         *c;
        double         *b;
        double         *A;
        double       *bfs;   
        double     *basis;
        double     *basin;
        int        basmod;
        int        augmod;
        double    *extcod;
        long       maxpiv;
        int          nBin;
        int          mBin;
        int         nrows;
        int         ncols;
        int        nrinvI[3];
        double     *nrinv;
        double     nclfac;

        nrinvI[1] = 0;
        nrinvI[2] = 0;

;

        nclfac   = (double) mxGetScalar(NCLFAC_IN);
        maxpiv   = (long)   mxGetScalar(MAXPIV);
        basmod   = (int)    mxGetScalar(MODE_IN);
        augmod   = (int)    mxGetScalar(AUGMODE);
;
;

      if ((nlhs != 5) & (nlhs != 6))
           mexErrMsgTxt("SIMPLEX requires 5 or 6 output arguments.");
      if (basmod != 2)
        if ((nrhs != 7) & (nrhs != 8)) 
           mexErrMsgTxt("SIMPLEX requires 7 or 8 input arguments.");
      if (basmod == 2)  
        if (nrhs != 7) mexErrMsgTxt("SIMPLEX requires 8 input arguments.");

;
;

        nBin      = mxGetN(B_IN);
        mBin      = mxGetM(B_IN);

 /*       BASIS_OUT = mxCreateFull(nBin,mBin,REAL);
        BFS_OUT   = mxCreateFull(mBin,nBin,REAL);
        Y_OUT     = mxCreateFull(nBin,mBin, REAL);
        N_OUT     = mxCreateFull(   1,   1, REAL);
        NREINVERT_OUT = mxCreateFull(   1,   2, REAL);
        EXITCODE  = mxCreateFull(   1,   1, REAL);
*/

        BASIS_OUT = mxCreateDoubleMatrix(nBin,mBin,mxREAL);
        BFS_OUT   = mxCreateDoubleMatrix(mBin,nBin,mxREAL);
        Y_OUT     = mxCreateDoubleMatrix(nBin,mBin, mxREAL);
        N_OUT     = mxCreateDoubleMatrix(   1,   1, mxREAL);
        NREINVERT_OUT = mxCreateDoubleMatrix(   1,   2, mxREAL);
        EXITCODE  = mxCreateDoubleMatrix(   1,   1, mxREAL);

        bfs       = mxGetPr(BFS_OUT);
        if (basmod==2) basin  = mxGetPr(BASIS_IN);
        basis     = mxGetPr(BASIS_OUT);
        y         = mxGetPr(Y_OUT);
        n         = mxGetPr(N_OUT);
        nrinv     = mxGetPr(NREINVERT_OUT);
        extcod    = mxGetPr(EXITCODE);
        c         = mxGetPr(C_IN);
        b         = mxGetPr(B_IN);
        A         = mxGetPr(A_IN);
        nrows     = mxGetM(A_IN);
        ncols     = mxGetN(A_IN);

        if (basmod == 1 && phase == 0)  
           mexErrMsgTxt("Augmentation mode invalid on first call.");

  mexAtExit(&freeup);

   srand(1);  
   *extcod = (double)simplx(basmod,&nI,bfs,basis,
                               y,nrows,ncols,c,b,A,basin,
                               augmod,maxpiv,nrinvI,nclfac); 

    *n         = (double) nI;
    nrinv[0] = (double) nrinvI[1];
    nrinv[1] = (double) nrinvI[2];

} 
