#!/bin/sh
# Example Script to compile and link a MEX file (relocatable DLL)
# File(s) that make up the DLL : localmax.c.
# Prerequisites:
# - GNU tools on the path
# - Availability of peclean.exe 
# (See http://www.cygnus.com/ml/gnu-win32/1997-Jul/0047.html
# A quick fix for the DLL .reloc problem)
#
# ***Fill in your path to libcygwin.a here (with no trailing slash)***
LIBPATH=/GNU_cdk/B19/H-i386-cygwin32/i386-cygwin32/lib
# ***Fill in your path to the matlab 5 root here (with no trailing slash)***
MATLAB=/Matlab5
# ***Fill in the complete path to the peclean executable ***
PECLEAN=/usr/local/bin/peclean


# Compile source file(s):
echo 'Compiling file(s) ...'
gcc -c -z -ansi -DMATLAB_MEX_FILE -I$MATLAB/extern/include simplexM.c
gcc -c -z -ansi -DMATLAB_MEX_FILE -I$MATLAB/extern/include simplex.c
gcc -c -z -ansi -DMATLAB_MEX_FILE -I$MATLAB/extern/include lu.c


# Create fixup.o which is needed to terminate the
# import list and the relocation section in the dll.
cat > fixup.c << EOF
/* Copied from winsup/dcrt0.cc in the cygwin32 source distribution. */
asm(".section .idata\$3\n" ".long 0,0,0,0, 0,0,0,0");
/* Next one is needed to properly terminate the .reloc section in the dll */
asm(".section .reloc\n" ".long 0,8");
EOF
gcc -c fixup.c


# Create a very minimalist startup routine for the dll.
# C copy of winsup/init.cc in the cygwin32 source distribution.
cat > init.c << EOF
#include <windows.h>


int WINAPI dll_entry (HANDLE h,
DWORD reason,
void *ptr)
{
switch (reason)
{
case DLL_PROCESS_ATTACH:
break;
case DLL_PROCESS_DETACH:
break;
case DLL_THREAD_ATTACH:
break;
case DLL_THREAD_DETACH:
break;
}
return 1;
}
EOF
gcc -c init.c


# Dlltool in the b18 release does not like .def files to start
# with a line 'LIBRARY <libname>'.
# Therefore this workaround: 
# extract library name from line 1 into variable libname
# and feed dlltool with a .def file with the 1st line stripped
# so the .def file starts with the line 'EXPORTS'
echo 'Creating MATLAB import library ...'
libname=`head -1 $MATLAB/extern/include/matlab.def | sed -e 's/^LIBRARY //'`
tail +2l $MATLAB/extern/include/matlab.def > temp.def
dlltool --def temp.def --dllname $libname --output-lib libmatlab.a
rm temp.def


# Make .def file:
echo EXPORTS > simplx.def
echo mexFunction >> simplx.def


# Link DLL.
echo 'Linking DLL ...'
ld --base-file simplx.base --dll -o simplx.dll lu.o simplex.o simplexm.o init.o fixup.o \
libmatlab.a $LIBPATH/libcygwin.a -e _dll_entry@12
dlltool --dllname simplx.dll --def simplx.def --base-file simplx.base \
--output-exp simplx.exp
ld simplx.exp --dll -o simplx.dll simplexm.o simplex.o lu.o init.o fixup.o \
libmatlab.a $LIBPATH/libcygwin.a -e _dll_entry@12


# Clean up
echo 'Cleaning up ...'
$PECLEAN simplex.dll
rm simplex.o simplexm.o lu.o init.c init.o fixup.c fixup.o libmatlab.a \
simplx.def simplx.exp simplx.base